@@ -17,7 +17,7 @@ use uv_build_frontend::{SourceBuild, SourceBuildContext};
 use uv_cache::Cache;
 use uv_client::RegistryClient;
 use uv_configuration::{
-    BuildKind, BuildOptions, Constraints, IndexStrategy, Preview, Reinstall, SourceStrategy,
+    BuildKind, BuildOptions, Constraints, IndexStrategy, Reinstall, SourceStrategy,
 };
 use uv_configuration::{BuildOutput, Concurrency};
 use uv_distribution::DistributionDatabase;
@@ -29,6 +29,7 @@ use uv_distribution_types::{
 };
 use uv_git::GitResolver;
 use uv_installer::{Installer, Plan, Planner, Preparer, SitePackages};
+use uv_preview::Preview;
 use uv_pypi_types::Conflicts;
 use uv_python::{Interpreter, PythonEnvironment};
 use uv_resolver::{
@@ -205,6 +206,10 @@ impl BuildContext for BuildDispatch<'_> {
         self.build_options
     }
 
+    fn build_isolation(&self) -> BuildIsolation<'_> {
+        self.build_isolation
+    }
+
     fn config_settings(&self) -> &ConfigSettings {
         self.config_settings
     }
@@ -480,7 +485,7 @@ impl BuildContext for BuildDispatch<'_> {
             self.workspace_cache(),
             config_settings,
             self.build_isolation,
-            self.extra_build_requires(),
+            self.extra_build_requires,
             &build_stack,
             build_kind,
             environment_variables,
