@@ -11,16 +11,19 @@ use uv_pep440::{Version, VersionSpecifiers};
 use uv_pep508::Requirement;
 
 use crate::lenient_requirement::LenientRequirement;
-use crate::metadata::pyproject_toml::PyProjectToml;
 use crate::metadata::Headers;
-use crate::{metadata, LenientVersionSpecifiers, MetadataError, VerbatimParsedUrl};
+use crate::metadata::pyproject_toml::PyProjectToml;
+use crate::{LenientVersionSpecifiers, MetadataError, VerbatimParsedUrl, metadata};
 
 /// A subset of the full core metadata specification, including only the
 /// fields that are relevant to dependency resolution.
 ///
 /// Core Metadata 2.3 is specified in <https://packaging.python.org/specifications/core-metadata/>.
-#[derive(Serialize, Deserialize, Debug, Clone)]
+#[derive(
+    Serialize, Deserialize, Debug, Clone, rkyv::Archive, rkyv::Deserialize, rkyv::Serialize,
+)]
 #[serde(rename_all = "kebab-case")]
+#[rkyv(derive(Debug))]
 pub struct ResolutionMetadata {
     // Mandatory fields
     pub name: PackageName,
