@@ -1,31 +1,28 @@
-use core::fmt;
-use fs_err as fs;
-
-use uv_configuration::Preview;
-use uv_dirs::user_executable_directory;
-use uv_pep440::Version;
-use uv_pep508::{InvalidNameError, PackageName};
-
 use std::io::{self, Write};
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
+use fs_err as fs;
 use fs_err::File;
 use thiserror::Error;
 use tracing::{debug, warn};
 
-use uv_install_wheel::read_record_file;
-
-pub use receipt::ToolReceipt;
-pub use tool::{Tool, ToolEntrypoint};
 use uv_cache::Cache;
+use uv_dirs::user_executable_directory;
 use uv_fs::{LockedFile, Simplified};
+use uv_install_wheel::read_record_file;
 use uv_installer::SitePackages;
+use uv_normalize::{InvalidNameError, PackageName};
+use uv_pep440::Version;
+use uv_preview::Preview;
 use uv_python::{Interpreter, PythonEnvironment};
 use uv_state::{StateBucket, StateStore};
 use uv_static::EnvVars;
 use uv_virtualenv::remove_virtualenv;
 
+pub use receipt::ToolReceipt;
+pub use tool::{Tool, ToolEntrypoint};
+
 mod receipt;
 mod tool;
 
@@ -352,8 +349,8 @@ impl InstalledTool {
     }
 }
 
-impl fmt::Display for InstalledTool {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+impl std::fmt::Display for InstalledTool {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(
             f,
             "{}",
